'use strict';

let BemCrud = require('bem-crud');
let _ = require('underscore');
let BemSchema = require('bem-schema');

class BemExtData {
  constructor(config) {
    config = config || {};
    this.db = config.db;
    this.schemaId = config.schemaId;
    //获取深度，默认最大取2层
    this.deep = config.deep || 2;
    if (Array.isArray(config.data)) {
      this.data = config.data;
    } else {
      this.data = [config.data];
    }
    this._deep = config._deep || 1;
  }

  * initSchemaData() {
    let crud = yield BemCrud.get({
      db: this.db,
      collection: '_schema'
    });
    let result = yield crud.read({
      query: {
        id: this.schemaId
      }
    });
    this._schema = result[0];
  }

  * buildControlMap() {
    this._controlMap = {};
    let fields = this._schema.fields;
    let controls = fields.map(function(item) {
      return item.control;
    });
    let crud = yield BemCrud.get({
      db: this.db,
      collection: '_control'
    });
    let result = yield crud.read({
      query: {
        name: {
          $in: controls
        }
      }
    });
    if (result.length > 0) {
      result.forEach((control) => {
        this._controlMap[control.name] = control;
      });
    }
  }

  * getFieldSchema(field) {
    let controlMap = this._controlMap;
    let controlData = controlMap[field.control];
    let collection = yield BemSchema.getCollectionBySchemaId(this.db, controlData.params.paramsValue.schema);
    let crud = yield BemCrud.get({
      db: this.db,
      collection: collection
    });
  }

  //没有关联其他数据的列，不需要转换
  needConvert(field) {
    let controlMap = this._controlMap;
    let controlData = controlMap[field.control];
    return controlData.params && controlData.params.paramsType === 'schema' && controlData.params.paramsValue && controlData.params.paramsValue.schema
  }

  getFieldsNeedConverted() {
    let fields = this._schema.fields;
    let needConverted = [];
    for (let i = 0; i < fields.length; i++) {
      let field = fields[i];
      if (this.needConvert(field)) {
        needConverted.push(field);
      }
    }
    return needConverted;
  }

  getFieldOptions(field) {
    let options = {
      valueField: '_id',
      displayField: 'name',
      params: {
        fields: {},
        sort: {}
      }
    };
    let controlMap = this._controlMap;
    let controlData = controlMap[field.control];
    let customParams = controlData.params.customParams;
    if (customParams) {
      if (typeof customParams === 'string') {
        try {
          customParams = JSON.parse(customParams);
        } catch (e) {
          customParams = {};
        }
      }
      Object.assign(options, customParams);
    }
    return options;
  }

  getFieldValues(field) {
    let data = this.data;
    let fieldValues = data.filter(function(doc) {
      return doc[field.field];
    }).map(function(doc) {
      return doc[field.field];
    });
    return fieldValues;
  }

  getUniqueFieldValues(field) {
    let fieldValues = this.getFieldValues(field);
    let allRowsValues = [];
    for (let i = 0; i < fieldValues.length; i++) {
      allRowsValues = allRowsValues.concat(fieldValues[i]);
    }
    let uniqueFieldValues = _.unique(allRowsValues);
    return uniqueFieldValues;
  }

  * joinExtData(field, result) {
    let db = this.db;
    let controlMap = this._controlMap;
    let controlData = controlMap[field.control];
    let schemaId = controlData.params.paramsValue.schema;
    let collection = yield BemSchema.getCollectionBySchemaId(this.db, schemaId);
    let fieldValues = this.getUniqueFieldValues(field);
    let fieldOptions = this.getFieldOptions(field);
    fieldOptions.params.query = {
      [fieldOptions.valueField]: {
        $in: fieldValues
      }
    }
    let crud = yield BemCrud.get({
      db, collection
    });
    let extData = (yield crud.read(fieldOptions.params));
    result[`${db}/${schemaId}`] = extData;
    let ext = new BemExtData({
      db: db,
      schemaId: schemaId,
      data: extData,
      deep: this.deep,
      _deep: this._deep + 1
    });
    yield ext.get({
      format: 'join',
      _joinResult: result
    });
  }

  * embedExtData(field) {
    let data = this.data;
    let controlMap = this._controlMap;
    let controlData = controlMap[field.control];
    let fieldValues = this.getFieldValues(field);
    let fieldOptions = this.getFieldOptions(field);
    let fieldExtData = [];
    let exist = {};
    let fieldSchemaMap = {};
    for (let j = 0; j < fieldValues.length; j++) {
      let fieldValue = fieldValues[j];
      fieldOptions.params.query = {
        [fieldOptions.valueField]: fieldValue
      };
      let isSingleValue = true;
      if (Array.isArray(fieldValue)) {
        isSingleValue = false;
        fieldOptions.params.query = {
          [fieldOptions.valueField]: {
            $in: fieldValue
          }
        }
      }

      let collection = yield BemSchema.getCollectionBySchemaId(this.db, controlData.params.paramsValue.schema);

      let fieldSchemaCrud = yield BemCrud.get({
        db: this.db,
        collection: collection
      });


      fieldExtData = yield fieldSchemaCrud.read(fieldOptions.params);

      if (fieldExtData && fieldExtData.length > 0) {
        if (Array.isArray(fieldValue)) {
          fieldExtData.sort(function(a, b) {
            let valueA = a[fieldOptions.valueField];
            let valueB = b[fieldOptions.valueField];
            //ObjectId
            if (typeof valueA === 'object') {
              valueA = valueA.toString();
            }
            //ObjectId
            if (typeof valueB === 'object') {
              valueB = valueB.toString();
            }
            return fieldValue.indexOf(valueA) - fieldValue.indexOf(valueB);
          });
        }

        let ext = new BemExtData({
          db: this.db,
          schemaId: controlData.params.paramsValue.schema,
          data: fieldExtData,
          deep: this.deep,
          _deep: this._deep + 1
        });
        let extData = yield ext.get({
          format: 'embed'
        });
        if (isSingleValue) {
          extData = extData[0];
        }
        fieldSchemaMap[fieldValue + ''] = extData;
      }
    }
    if (Object.keys(fieldSchemaMap).length > 0) {
      for (let j = 0; j < data.length; j++) {
        data[j][field.field] = fieldSchemaMap[data[j][field.field] + ''];
      }
    }
  }

  * get(options) {
    let format = options.format || 'join';
    let joinResult = options._joinResult || {};
    if (this._deep <= this.deep) {
      yield this.initSchemaData();
      if (this._schema) {
        yield this.buildControlMap();
        let controlMap = this._controlMap;
        let fieldsNeedConverted = this.getFieldsNeedConverted();
        for (let i = 0; i < fieldsNeedConverted.length; i++) {
          let field = fieldsNeedConverted[i];
          if (format === 'join') {
            yield this.joinExtData(field, joinResult);
          } else {
            yield this.embedExtData(field);
          }
        }
      }
    }
    return format === 'join' ? joinResult : this.data;
  }
}

module.exports = BemExtData;
