'use strict'

let assert = require("assert");
let BemExtData = require('./');
let thunkify = require('thunkify');
let co = require('co');
let BemCrud = require('bem-crud');

describe('BemExtData', function() {
  let user1, user2, ext;
  before(function(done) {
    co(function*() {
      let dataCrud = yield BemCrud.get({
        db: 'test',
        collection: 'data'
      });
      let extCrud = yield BemCrud.get({
        db: 'test',
        collection: 'ext'
      });
      let schemaCrud = yield BemCrud.get({
        db: 'test',
        collection: '_schema'
      });
      let controlCrud = yield BemCrud.get({
        db: 'test',
        collection: '_control'
      });
      let userCrud = yield BemCrud.get({
        db: 'test',
        collection: '_user'
      });

      yield dataCrud.delete({});
      yield extCrud.delete({});
      yield schemaCrud.delete({});
      yield controlCrud.delete({});
      yield userCrud.delete({});

      yield schemaCrud.create({
        docs: {
          "id": "data",
          "collection": "data",
          "params": {
            "create": true,
            "update": true,
            "remove": true,
            "multiRemove": true
          },
          "fields": [{
            "show": "yes",
            "edit": "yes",
            "field": "ext",
            "name": "ext",
            "control": "draggable-selector-ext"
          }]
        }
      });

      yield schemaCrud.create({
        docs: {
          "id": "ext",
          "collection": "ext",
          "params": {
            "create": true,
            "update": true,
            "remove": true,
            "multiRemove": true
          },
          "fields": [{
            "show": "yes",
            "edit": "yes",
            "field": "users",
            "name": "users",
            "control": "draggable-selector-user"
          }]
        }
      });

      yield schemaCrud.create({
        docs: {
          "id": "_user",
          "collection": "_user",
          "params": {
            "create": true,
            "update": true,
            "remove": true,
            "multiRemove": true
          },
          "fields": [{
            "show": "yes",
            "edit": "yes",
            "field": "users",
            "name": "users",
            "control": "draggable-selector-user"
          }]
        }
      });

      yield controlCrud.create({
        docs: {
          "name": "draggable-selector-ext",
          "base": "draggable-selector",
          "params": {
            "paramsType": "schema",
            "paramsValue": {
              "schema": "ext"
            }
          }
        }
      });

      yield controlCrud.create({
        docs: {
          "name": "draggable-selector-user",
          "base": "draggable-selector",
          "params": {
            "paramsType": "schema",
            "paramsValue": {
              "schema": "_user"
            },
            "customParams": JSON.stringify({
              "displayField": "nickname",
              "params": {
                "fields": {
                  "name": 1
                }
              }
            })
          }
        }
      });

      user1 = yield userCrud.create({
        docs: {
          name: 'seon',
          age: 17
        }
      });

      user2 = yield userCrud.create({
        docs: {
          name: 'viviann',
          age: 16
        }
      });

      ext = yield extCrud.create({
        docs: {
          users: [user2._id.toString(), user1._id.toString()]
        }
      });

      yield dataCrud.create({
        docs: {
          ext: [ext._id.toString()]
        }
      });

    }).then(done, done);
  });
  describe('#get', function() {
    it('get ext data with format embed', function(done) {
      co(function*() {
        let dataCrud = yield BemCrud.get({
          db: 'test',
          collection: 'data'
        });
        let data = yield dataCrud.read({});
        let extData = new BemExtData({
          db: 'test',
          schemaId: 'data',
          data: data
        });
        let ext = yield extData.get({
          format: 'embed'
        });
        //保证顺序与原来一致
        assert.equal(ext[0].ext[0].users[0].name, 'viviann');
        assert.equal(ext[0].ext[0].users[1].name, 'seon');
        //field指定了不返回age字段
        assert.equal(ext[0].ext[0].users.age, null);
      }).then(done, done);
    });

    it('get ext data with format embed, max deep is 1', function(done) {
      co(function*() {
        let dataCrud = yield BemCrud.get({
          db: 'test',
          collection: 'data'
        });
        let data = yield dataCrud.read({});
        let extData = new BemExtData({
          db: 'test',
          schemaId: 'data',
          data: data,
          deep: 1
        });
        let ext = yield extData.get({
          format: 'embed'
        });
        //保证顺序与原来一致
        assert.equal(ext[0].ext[0].users[0], user2._id.toString());
        assert.equal(ext[0].ext[0].users[1], user1._id.toString());
        //field指定了不返回age字段
        assert.equal(ext[0].ext[0].users.age, null);
      }).then(done, done);
    });

    it('get ext data with format join', function(done) {
      co(function*() {
        let dataCrud = yield BemCrud.get({
          db: 'test',
          collection: 'data'
        });
        let data = yield dataCrud.read({});
        let extData = new BemExtData({
          db: 'test',
          schemaId: 'data',
          data: data
        });
        let ext = yield extData.get({});
        assert.equal(ext['test/ext'][0].users[0], user2._id.toString());
        assert.equal(ext['test/ext'][0].users[1], user1._id.toString());
        assert.equal(ext['test/_user'][0].name, 'seon');
        assert.equal(ext['test/_user'][0].age, null);
      }).then(done, done);
    });

  });
});
